# /etc/rc.S
#
# These commands are executed at boot time by init(8).
# User customization should go in /etc/rc.local.

PATH=/sbin:/usr/sbin:/bin:/usr/bin

# enable swapping
/sbin/swapon -a

# Start update.
/sbin/update &

# Test to see if the root partition is read-only, like it ought to be.
READWRITE=no
if echo -n >> "Teste filesystem status"; then
 rm -f "Teste filesystem status"
 READWRITE=yes
fi

# Check the integrity of all filesystems
if [ ! $READWRITE = yes ]; then
 /sbin/fsck -A -a
 # If there was a failure, drop into single-user mode.
 if [ $? -gt 1 ] ; then
  echo
  echo
  echo "**************************************"
  echo "fsck returned error code - REBOOT NOW!"
  echo "**************************************"
  echo
  echo
  /bin/login 
 fi
 # Remount the root filesystem in read-write mode
 echo "Remounting root device with read-write enabled."
 /sbin/mount -w -n -o remount /
else
 cat << EOF

*** Ihre Rootpartition ist bereits read-write gemountet ! Kann keinen 
Filesystem Check durchfhren.
Damit der Filesystem-Check richtig funktioniert, mssen Sie die Rootpartition
zuerst read only mounten. Bitte ndern Sie Ihren Kernel mit 'rdev -R'.
Wenn Sie mit LILO booten, so geben Sie ein:

rdev -R /IHR_KERNEL 1 

EOF
 sleep 10
fi

# remove /etc/mtab* so that mount will create it with a root entry
/bin/rm -f /etc/mtab* /etc/nologin /etc/utmp

# Looks like we have to create this.
cat /dev/null >> /etc/utmp

# mount file systems in fstab (and create an entry for /)
# but not NFS because TCP/IP is not yet configured
/sbin/mount -avt nonfs

# Loading german keytable
if [ -f /usr/lib/kbd/keytables/gr-lat1.map ]; then
	echo "Lade deutsche Tastatur..."
	/bin/loadkeys /usr/lib/kbd/keytables/gr-lat1.map
fi

# Configure the system clock. See /usr/doc/ktzset. Replaces /sbin/clock -s
if [ -x /sbin/kern_tzset ]; then
  /sbin/kern_tzset -u
fi

if [ -f /etc/.lst ]; then
	cat /etc/.lst > /etc/issue
fi

if [ -x /usr/local/bin/pm ]; then
	/usr/local/bin/pm > /etc/motd
fi

# Remember to call syssetup -install at least once
if [ -f /etc/syssetup.notice ]; then
        cat /etc/syssetup.notice >> /etc/issue
fi

# Remember to call netsetup -install at least once
if [ -f /etc/netsetup.notice -a ! -f /etc/syssetup.notice ]; then
        cat /etc/netsetup.notice >> /etc/issue
fi

# Set up the serial ports.
# This made a couple machines I tested this on hang at this point in the boot
# process, so it's getting commented out by default. However, if you need to
# set up extended serial ports try it out. It usually works great.
#/bin/sh /etc/rc.d/rc.serial

